@@ -38,8 +38,9 @@ import libarchive
 
 
 from . import conda_interface, utils
-from .conda_interface import MatchSpec, VersionOrder, human_bytes
+from .conda_interface import MatchSpec, VersionOrder, human_bytes, context
 from .conda_interface import CondaError, CondaHTTPError, get_index, url_path
+from .conda_interface import download, TemporaryDirectory
 from .utils import glob, get_logger, FileNotFoundError, PermissionError
 
 try:
@@ -131,6 +132,8 @@ local_index_timestamp = 0
 cached_index = None
 local_subdir = ""
 cached_channels = []
+channel_data = {}
+
 
 MAX_THREADS_DEFAULT = os.cpu_count() if (hasattr(os, "cpu_count") and os.cpu_count() > 1) else 1
 LOCK_TIMEOUT_SECS = 3 * 3600
@@ -159,6 +162,18 @@ except ImportError:  # pragma: no cover
     from conda._vendor.toolz.itertoolz import concat, concatv, groupby  # NOQA
 
 
+def _download_channeldata(channel_url):
+    with TemporaryDirectory() as td:
+        tf = os.path.join(td, "channeldata.json")
+        download(channel_url, tf)
+        try:
+            with open(tf) as f:
+                data = json.load(f)
+        except json.decoder.JSONDecodeError:
+            data = {}
+    return data
+
+
 def get_build_index(subdir, bldpkgs_dir, output_folder=None, clear_cache=False,
                     omit_defaults=False, channel_urls=None, debug=False, verbose=True,
                     **kwargs):
@@ -166,6 +181,7 @@ def get_build_index(subdir, bldpkgs_dir, output_folder=None, clear_cache=False,
     global local_subdir
     global cached_index
     global cached_channels
+    global channel_data
     mtime = 0
 
     channel_urls = list(utils.ensure_list(channel_urls))
@@ -233,10 +249,46 @@ def get_build_index(subdir, bldpkgs_dir, output_folder=None, clear_cache=False,
                                              use_local=False,
                                              use_cache=False,
                                              platform=subdir)
+
+            expanded_channels = {rec.channel for rec in cached_index.values()}
+
+            superchannel = {}
+            # we need channeldata.json too, as it is a more reliable source of run_exports data
+            for channel in expanded_channels:
+                if channel.scheme == "file":
+                    location = channel.location
+                    if utils.on_win:
+                        location = location.lstrip("/")
+                    elif (not os.path.isabs(channel.location) and
+                            os.path.exists(os.path.join(os.path.sep, channel.location))):
+                        location = os.path.join(os.path.sep, channel.location)
+                    channeldata_file = os.path.join(location, channel.name, 'channeldata.json')
+                    retry = 0
+                    max_retries = 10
+                    while retry < max_retries:
+                        try:
+                            with open(channeldata_file, "r+") as f:
+                                channel_data[channel.name] = json.load(f)
+                            break
+                        except (IOError, json.decoder.JSONDecodeError):
+                            time.sleep(0.2)
+                            retry += 1
+                else:
+                    # download channeldata.json for url
+                    try:
+                        channel_data[channel.name] = _download_channeldata(channel.base_url + '/channeldata.json')
+                    except CondaHTTPError:
+                        continue
+                # collapse defaults metachannel back into one superchannel, merging channeldata
+                if channel.base_url in context.default_channels and channel_data.get(channel.name):
+                    packages = superchannel.get('packages', {})
+                    packages.update(channel_data[channel.name])
+                    superchannel['packages'] = packages
+            channel_data['defaults'] = superchannel
         local_index_timestamp = os.path.getmtime(index_file)
         local_subdir = subdir
         cached_channels = channel_urls
-    return cached_index, local_index_timestamp
+    return cached_index, local_index_timestamp, channel_data
 
 
 def _ensure_valid_channel(local_folder, subdir):
@@ -277,7 +329,7 @@ def _determine_namespace(info):
         namespace = info['namespace']
     else:
         depends_names = set()
-        for spec in info['depends']:
+        for spec in info.get('depends', []):
             try:
                 depends_names.add(MatchSpec(spec).name)
             except CondaError:
@@ -355,7 +407,8 @@ def _clear_newline_chars(record, field_name):
 
 def _apply_instructions(subdir, repodata, instructions):
     repodata.setdefault("removed", [])
-    utils.merge_or_update_dict(repodata.get('packages', {}), instructions.get('packages', {}), merge=False)
+    utils.merge_or_update_dict(repodata.get('packages', {}), instructions.get('packages', {}), merge=False,
+                               add_missing_keys=False)
 
     for fn in instructions.get('revoke', ()):
         repodata['packages'][fn]['revoked'] = True
@@ -598,13 +651,13 @@ def _augment_repodata(subdirs, patched_repodata, patch_instructions):
                     info['depends2'] = [_add_namespace_to_spec(fn, info, dep, namemap, missing_dependencies, subdir)
                                         for dep in info['depends'] if dep.split()[0] not in constrains_names]
                 except CondaError as e:
-                    log.warn("Encountered a file that conda does not like.  Error was: {}.  Skipping this one...".format(fn))
+                    log.warn("Encountered a file ({}) that conda does not like.  Error was: {}.  Skipping this one...".format(fn, e))
             else:
                 try:
                     info['depends2'] = [_add_namespace_to_spec(fn, info, dep, namemap, missing_dependencies, subdir)
                                         for dep in info['depends']]
                 except CondaError as e:
-                    log.warn("Encountered a file that conda does not like.  Error was: {}.  Skipping this one...".format(fn))
+                    log.warn("Encountered a file ({}) that conda does not like.  Error was: {}.  Skipping this one...".format(fn, e))
             # info['build_string'] =_make_build_string(info["build"], info["build_number"])
         repodata["removed"] = patch_instructions[subdir].get("remove", [])
         augmented_repodata[subdir] = repodata
@@ -690,17 +743,33 @@ def _cache_recipe_log(tar_path, recipe_log_path):
         fh.write(binary_recipe_log)
 
 
-def _cache_run_exports(tar_path, run_exports_cache_path):
-    try:
-        binary_run_exports = _tar_xf_file(tar_path, 'info/run_exports.json')
-        run_exports = json.loads(binary_run_exports.decode("utf-8"))
-    except KeyError:
+def get_run_exports(tar_or_folder_path):
+    run_exports = {}
+    if os.path.isfile(tar_or_folder_path):
         try:
-            binary_run_exports = _tar_xf_file(tar_path, 'info/run_exports.yaml')
-            run_exports = yaml.safe_load(binary_run_exports)
+            binary_run_exports = _tar_xf_file(tar_or_folder_path, 'info/run_exports.json')
+            run_exports = json.loads(binary_run_exports.decode("utf-8"))
         except KeyError:
-            log.debug("%s has no run_exports file (this is OK)" % tar_path)
-            run_exports = {}
+            try:
+                binary_run_exports = _tar_xf_file(tar_or_folder_path, 'info/run_exports.yaml')
+                run_exports = yaml.safe_load(binary_run_exports)
+            except KeyError:
+                log.debug("%s has no run_exports file (this is OK)" % tar_or_folder_path)
+    elif os.path.isdir(tar_or_folder_path):
+        try:
+            with open(os.path.join(tar_or_folder_path, 'info', 'run_exports.json')) as f:
+                run_exports = json.load(f)
+        except (IOError, FileNotFoundError):
+            try:
+                with open(os.path.join(tar_or_folder_path, 'info', 'run_exports.yaml')) as f:
+                    run_exports = yaml.safe_load(f)
+            except (IOError, FileNotFoundError):
+                log.debug("%s has no run_exports file (this is OK)" % tar_or_folder_path)
+    return run_exports
+
+
+def _cache_run_exports(tar_path, run_exports_cache_path):
+    run_exports = get_run_exports(tar_path)
     with open(run_exports_cache_path, 'w') as fh:
         json.dump(run_exports, fh)
 
@@ -1287,7 +1356,7 @@ class ChannelIndex(object):
 
     def _write_channeldata(self, channeldata):
         # trim out commits, as they can take up a ton of space.  They're really only for the RSS feed.
-        for pkg, pkg_dict in channeldata.get('packages', {}).items():
+        for _pkg, pkg_dict in channeldata.get('packages', {}).items():
             if "commits" in pkg_dict:
                 del pkg_dict['commits']
         channeldata_path = join(self.channel_root, 'channeldata.json')
