@@ -64,7 +64,7 @@ def bldpkg_path(m):
     if not hasattr(m, 'type') or m.type == "conda":
         path = os.path.join(m.config.output_folder, subdir, '%s%s' % (m.dist(), CONDA_TARBALL_EXTENSIONS[0]))
     else:
-        path = '{} file for {} in: {} '.format(m.type, m.name(), os.path.join(m.config.output_folder, subdir))
+        path = '{} file for {} in: {}'.format(m.type, m.name(), os.path.join(m.config.output_folder, subdir))
     return path
 
 
@@ -79,24 +79,11 @@ def actions_to_pins(actions):
     return specs
 
 
-def get_env_dependencies(m, env, variant, exclude_pattern=None,
-                         permit_unsatisfiable_variants=False,
-                         merge_build_host_on_same_platform=True):
-    dash_or_under = re.compile("[-_]")
-    specs = m.get_depends_top_and_out(env)
-    # replace x.x with our variant's numpy version, or else conda tries to literally go get x.x
-    if env in ('build', 'host'):
-        no_xx_specs = []
-        for spec in specs:
-            if ' x.x' in spec:
-                pkg_name = spec.split()[0]
-                no_xx_specs.append(' '.join((pkg_name, variant.get(pkg_name, ""))))
-            else:
-                no_xx_specs.append(spec)
-        specs = no_xx_specs
+def _categorize_deps(m, specs, exclude_pattern, variant):
     subpackages = []
     dependencies = []
     pass_through_deps = []
+    dash_or_under = re.compile("[-_]")
     # ones that get filtered from actual versioning, to exclude them from the hash calculation
     for spec in specs:
         if not exclude_pattern or not exclude_pattern.match(spec):
@@ -117,10 +104,30 @@ def get_env_dependencies(m, env, variant, exclude_pattern=None,
                     dependencies.append(" ".join((spec_name, value)))
         elif exclude_pattern.match(spec):
             pass_through_deps.append(spec)
-    random_string = ''.join(random.choice(string.ascii_uppercase + string.digits)
-                            for _ in range(10))
+    return subpackages, dependencies, pass_through_deps
+
+
+def get_env_dependencies(m, env, variant, exclude_pattern=None,
+                         permit_unsatisfiable_variants=False,
+                         merge_build_host_on_same_platform=True):
+    specs = m.get_depends_top_and_out(env)
+    # replace x.x with our variant's numpy version, or else conda tries to literally go get x.x
+    if env in ('build', 'host'):
+        no_xx_specs = []
+        for spec in specs:
+            if ' x.x' in spec:
+                pkg_name = spec.split()[0]
+                no_xx_specs.append(' '.join((pkg_name, variant.get(pkg_name, ""))))
+            else:
+                no_xx_specs.append(spec)
+        specs = no_xx_specs
+
+    subpackages, dependencies, pass_through_deps = _categorize_deps(m, specs, exclude_pattern, variant)
+
     dependencies = set(dependencies)
     unsat = None
+    random_string = ''.join(random.choice(string.ascii_uppercase + string.digits)
+                            for _ in range(10))
     with TemporaryDirectory(prefix="_", suffix=random_string) as tmpdir:
         try:
             actions = environ.get_install_actions(tmpdir, tuple(dependencies), env,
@@ -160,7 +167,11 @@ def strip_channel(spec_str):
 
 
 def get_pin_from_build(m, dep, build_dep_versions):
-    dep_name = dep.split()[0]
+    dep_split = dep.split()
+    dep_name = dep_split[0]
+    build = ''
+    if len(dep_split) >= 3:
+        build = dep_split[2]
     pin = None
     version = build_dep_versions.get(dep_name) or m.config.variant.get(dep_name)
     if (version and dep_name in m.config.variant.get('pin_run_as_build', {}) and
@@ -176,7 +187,7 @@ def get_pin_from_build(m, dep, build_dep_versions):
             raise ValueError("numpy x.x specified, but numpy not in build requirements.")
         pin = utils.apply_pin_expressions(version.split()[0], min_pin='x.x', max_pin='x.x')
     if pin:
-        dep = " ".join((dep_name, pin))
+        dep = " ".join((dep_name, pin, build)).strip()
     return dep
 
 
@@ -259,12 +270,10 @@ def _read_specs_from_package(pkg_loc, pkg_dist):
 
 
 def execute_download_actions(m, actions, env, package_subset=None, require_files=False):
-    index, index_ts = get_build_index(getattr(m.config, '{}_subdir'.format(env)),
-                                      bldpkgs_dir=m.config.bldpkgs_dir,
-                                      output_folder=m.config.output_folder,
-                                      channel_urls=m.config.channel_urls,
-                                      debug=m.config.debug, verbose=m.config.verbose,
-                                      locking=m.config.locking, timeout=m.config.timeout)
+    index, _, _ = get_build_index(getattr(m.config, '{}_subdir'.format(env)), bldpkgs_dir=m.config.bldpkgs_dir,
+                                  output_folder=m.config.output_folder, channel_urls=m.config.channel_urls,
+                                  debug=m.config.debug, verbose=m.config.verbose, locking=m.config.locking,
+                                  timeout=m.config.timeout)
 
     # this should be just downloading packages.  We don't need to extract them -
     #    we read contents directly
@@ -389,8 +398,9 @@ def add_upstream_pins(m, permit_unsatisfiable_variants, exclude_pattern):
         host_deps = []
         host_unsat = []
         extra_run_specs = set(extra_run_specs_from_build.get('strong', []))
-        if not m.uses_new_style_compiler_activation and not m.build_is_host:
+        if m.build_is_host:
             extra_run_specs.update(extra_run_specs_from_build.get('weak', []))
+            build_deps = set(build_deps or []).update(extra_run_specs_from_build.get('weak', []))
         else:
             host_deps = set(extra_run_specs_from_build.get('strong', []))
 
@@ -424,7 +434,12 @@ def _simplify_to_exact_constraints(metadata):
 
         deps_list = []
         for name, values in deps_dict.items():
-            exact_pins = [dep for dep in values if len(dep) > 1]
+            exact_pins = []
+            for dep in values:
+                if len(dep) > 1:
+                    version, build = dep[:2]
+                    if not (any(c in version for c in ('>', '<', '*')) or '*' in build):
+                        exact_pins.append(dep)
             if len(values) == 1 and not any(values):
                 deps_list.append(name)
             elif exact_pins:
@@ -439,12 +454,15 @@ def _simplify_to_exact_constraints(metadata):
     metadata.meta['requirements'] = requirements
 
 
-def finalize_metadata(m, permit_unsatisfiable_variants=False):
+def finalize_metadata(m, parent_metadata=None, permit_unsatisfiable_variants=False):
     """Fully render a recipe.  Fill in versions for build/host dependencies."""
+    if not parent_metadata:
+        parent_metadata = m
     if m.skip():
         rendered_metadata = m.copy()
         rendered_metadata.final = True
     else:
+
         exclude_pattern = None
         excludes = set(m.config.variant.get('ignore_version', []))
 
@@ -465,8 +483,9 @@ def finalize_metadata(m, permit_unsatisfiable_variants=False):
         # extract the topmost section where variables are defined, and put it on top of the
         #     requirements for a particular output
         # Re-parse the output from the original recipe, so that we re-consider any jinja2 stuff
-        output = m.get_rendered_output(m.name())
-        rendered_metadata = m.get_output_metadata(output)
+        parent_metadata = parent_metadata.copy()
+        parent_metadata.config.variant = m.config.variant
+        output = parent_metadata.get_rendered_output(m.name())
 
         if output:
             if 'package' in output or 'name' not in output:
@@ -474,22 +493,32 @@ def finalize_metadata(m, permit_unsatisfiable_variants=False):
                 output = {'name': m.name()}
 
             if not parent_recipe or parent_recipe['name'] == m.name():
-                combine_top_level_metadata_with_output(rendered_metadata, output)
+                combine_top_level_metadata_with_output(m, output)
             requirements = utils.expand_reqs(output.get('requirements', {}))
-            rendered_metadata.meta['requirements'] = requirements
+            m.meta['requirements'] = requirements
 
-        if rendered_metadata.meta.get('requirements'):
-            utils.insert_variant_versions(rendered_metadata.meta['requirements'],
-                                          rendered_metadata.config.variant, 'build')
-            utils.insert_variant_versions(rendered_metadata.meta['requirements'],
-                                        rendered_metadata.config.variant, 'host')
+        if m.meta.get('requirements'):
+            utils.insert_variant_versions(m.meta['requirements'],
+                                          m.config.variant, 'build')
+            utils.insert_variant_versions(m.meta['requirements'],
+                                        m.config.variant, 'host')
 
-        build_unsat, host_unsat = add_upstream_pins(rendered_metadata,
+        m = parent_metadata.get_output_metadata(m.get_rendered_output(m.name()))
+        build_unsat, host_unsat = add_upstream_pins(m,
                                                     permit_unsatisfiable_variants,
                                                     exclude_pattern)
         # getting this AFTER add_upstream_pins is important, because that function adds deps
         #     to the metadata.
-        requirements = rendered_metadata.meta.get('requirements', {})
+        requirements = m.meta.get('requirements', {})
+
+        # this is hacky, but it gets the jinja2 things like pin_compatible from the rendered output
+        # rerendered_output = parent_metadata.get_output_metadata(m.get_rendered_output(m.name()))
+        # run_reqs = utils.expand_reqs(rerendered_output.meta.get('requirements', {}))
+        # run_reqs = run_reqs.get('run', [])
+        # if run_reqs:
+        #     requirements['run'] = run_reqs
+        # m.meta['requirements'] = requirements
+        # m.meta['build'] = rerendered_output.meta.get('build', {})
 
         # here's where we pin run dependencies to their build time versions.  This happens based
         #     on the keys in the 'pin_run_as_build' key in the variant, which is a list of package
@@ -497,17 +526,17 @@ def finalize_metadata(m, permit_unsatisfiable_variants=False):
         if output_excludes:
             exclude_pattern = re.compile(r'|'.join(r'(?:^{}(?:\s|$|\Z))'.format(exc)
                                             for exc in output_excludes))
-        pinning_env = 'host' if rendered_metadata.is_cross else 'build'
+        pinning_env = 'host' if m.is_cross else 'build'
 
         build_reqs = requirements.get(pinning_env, [])
         # if python is in the build specs, but doesn't have a specific associated
         #    version, make sure to add one
         if build_reqs and 'python' in build_reqs:
             build_reqs.append('python {}'.format(m.config.variant['python']))
-            rendered_metadata.meta['requirements'][pinning_env] = build_reqs
+            m.meta['requirements'][pinning_env] = build_reqs
 
-        full_build_deps, _, _ = get_env_dependencies(rendered_metadata, pinning_env,
-                                        rendered_metadata.config.variant,
+        full_build_deps, _, _ = get_env_dependencies(m, pinning_env,
+                                        m.config.variant,
                                         exclude_pattern=exclude_pattern,
                                         permit_unsatisfiable_variants=permit_unsatisfiable_variants)
         full_build_dep_versions = {dep.split()[0]: " ".join(dep.split()[1:])
@@ -517,31 +546,31 @@ def finalize_metadata(m, permit_unsatisfiable_variants=False):
             requirements['run'] = specs_from_url(m.requirements_path)
         run_deps = requirements.get('run', [])
 
-        versioned_run_deps = [get_pin_from_build(rendered_metadata, dep, full_build_dep_versions)
+        versioned_run_deps = [get_pin_from_build(m, dep, full_build_dep_versions)
                             for dep in run_deps]
         versioned_run_deps = [utils.ensure_valid_spec(spec, warn=True)
                               for spec in versioned_run_deps]
         requirements[pinning_env] = full_build_deps
         requirements['run'] = versioned_run_deps
 
-        rendered_metadata.meta['requirements'] = requirements
+        m.meta['requirements'] = requirements
 
         # append other requirements, such as python.app, appropriately
-        rendered_metadata.append_requirements()
+        m.append_requirements()
 
-        if rendered_metadata.pin_depends == 'strict':
-            rendered_metadata.meta['requirements']['run'] = environ.get_pinned_deps(
-                rendered_metadata, 'run')
-        test_deps = rendered_metadata.get_value('test/requires')
+        if m.pin_depends == 'strict':
+            m.meta['requirements']['run'] = environ.get_pinned_deps(
+                m, 'run')
+        test_deps = m.get_value('test/requires')
         if test_deps:
             versioned_test_deps = list({get_pin_from_build(m, dep, full_build_dep_versions)
                                         for dep in test_deps})
             versioned_test_deps = [utils.ensure_valid_spec(spec, warn=True)
                                 for spec in versioned_test_deps]
-            rendered_metadata.meta['test']['requires'] = versioned_test_deps
-        extra = rendered_metadata.meta.get('extra', {})
+            m.meta['test']['requires'] = versioned_test_deps
+        extra = m.meta.get('extra', {})
         extra['copy_test_source_files'] = m.config.copy_test_source_files
-        rendered_metadata.meta['extra'] = extra
+        m.meta['extra'] = extra
 
         # if source/path is relative, then the output package makes no sense at all.  The next
         #   best thing is to hard-code the absolute path.  This probably won't exist on any
@@ -551,33 +580,33 @@ def finalize_metadata(m, permit_unsatisfiable_variants=False):
                 source_path = m.meta['source']['path']
                 os.path.expanduser(source_path)
                 if not os.path.isabs(source_path):
-                    rendered_metadata.meta['source']['path'] = os.path.normpath(
+                    m.meta['source']['path'] = os.path.normpath(
                         os.path.join(m.path, source_path))
                 elif ('git_url' in m.meta['source'] and not (
                         # absolute paths are not relative paths
                         os.path.isabs(m.meta['source']['git_url']) or
                         # real urls are not relative paths
                         ":" in m.meta['source']['git_url'])):
-                    rendered_metadata.meta['source']['git_url'] = os.path.normpath(
+                    m.meta['source']['git_url'] = os.path.normpath(
                         os.path.join(m.path, m.meta['source']['git_url']))
 
-        if not rendered_metadata.meta.get('build'):
-            rendered_metadata.meta['build'] = {}
+        if not m.meta.get('build'):
+            m.meta['build'] = {}
 
-        _simplify_to_exact_constraints(rendered_metadata)
+        _simplify_to_exact_constraints(m)
 
         if build_unsat or host_unsat:
-            rendered_metadata.final = False
+            m.final = False
             log = utils.get_logger(__name__)
             log.warn("Returning non-final recipe for {}; one or more dependencies "
-                    "was unsatisfiable:".format(rendered_metadata.dist()))
+                    "was unsatisfiable:".format(m.dist()))
             if build_unsat:
                 log.warn("Build: {}".format(build_unsat))
             if host_unsat:
                 log.warn("Host: {}".format(host_unsat))
         else:
-            rendered_metadata.final = True
-    return rendered_metadata
+            m.final = True
+    return m
 
 
 def try_download(metadata, no_download_source, raise_error=False):
@@ -703,7 +732,7 @@ def expand_outputs(metadata_tuples):
     expanded_outputs = OrderedDict()
 
     for (_m, download, reparse) in metadata_tuples:
-        for (output_dict, m) in _m.get_output_metadata_set(permit_unsatisfiable_variants=False):
+        for (output_dict, m) in _m.copy().get_output_metadata_set(permit_unsatisfiable_variants=False):
             expanded_outputs[m.dist()] = (output_dict, m)
     return list(expanded_outputs.values())
 
